package com.sun.media.sound;

import javax.sound.midi.MidiEvent;
import javax.sound.midi.MetaMessage;
import javax.sound.midi.SysexMessage;
import javax.sound.midi.Track;
import javax.sound.midi.InvalidMidiDataException;
import java.io.EOFException;
import java.io.IOException;
import java.io.DataInputStream;

final class SMFParser
{
    private static final int MTrk_MAGIC = 1297379947;
    private static final boolean STRICT_PARSER = false;
    private static final boolean DEBUG = false;
    int tracks;
    DataInputStream stream;
    private int trackLength;
    private byte[] trackData;
    private int pos;
    
    SMFParser() {
        this.trackLength = 0;
        this.trackData = null;
        this.pos = 0;
    }
    
    private int readUnsigned() throws IOException {
        return this.trackData[this.pos++] & 0xFF;
    }
    
    private void read(byte[] array) throws IOException {
        System.arraycopy(this.trackData, this.pos, array, 0, array.length);
        this.pos += array.length;
    }
    
    private long readVarInt() throws IOException {
        long n = 0L;
        int n2;
        do {
            n2 = (this.trackData[this.pos++] & 0xFF);
            n = (n << 7) + (long)(n2 & 0x7F);
        } while ((n2 & 0x80) != 0x0);
        return n;
    }
    
    private int readIntFromStream() throws IOException {
        try {
            return this.stream.readInt();
        }
        catch (EOFException ex) {
            throw new EOFException("invalid MIDI file");
        }
    }
    
    boolean nextTrack() throws IOException, InvalidMidiDataException {
        this.trackLength = 0;
        while (this.stream.skipBytes(this.trackLength) == this.trackLength) {
            int intFromStream = this.readIntFromStream();
            this.trackLength = this.readIntFromStream();
            if (intFromStream != 1297379947) {
                continue;
            }
            if (this.trackLength < 0) {
                return false;
            }
            try {
                this.trackData = new byte[this.trackLength];
            }
            catch (OutOfMemoryError cause) {
                throw new IOException("Track length too big", cause);
            }
            try {
                this.stream.readFully(this.trackData);
            }
            catch (EOFException ex) {
                return false;
            }
            this.pos = 0;
            return true;
        }
        return false;
    }
    
    private boolean trackFinished() {
        return this.pos >= this.trackLength;
    }
    
    void readTrack(Track track) throws IOException, InvalidMidiDataException {
        try {
            long tick = 0L;
            int i = 0;
            int n = 0;
            while (!this.trackFinished() && n == 0) {
                int n2 = -1;
                tick += this.readVarInt();
                int unsigned = this.readUnsigned();
                if (unsigned >= 128) {
                    i = unsigned;
                }
                else {
                    n2 = unsigned;
                }
                Object message;
                switch (i & 0xF0) {
                    case 128:
                    case 144:
                    case 160:
                    case 176:
                    case 224: {
                        if (n2 == -1) {
                            n2 = this.readUnsigned();
                        }
                        message = new FastShortMessage(i | n2 << 8 | this.readUnsigned() << 16);
                        break;
                    }
                    case 192:
                    case 208: {
                        if (n2 == -1) {
                            n2 = this.readUnsigned();
                        }
                        message = new FastShortMessage(i | n2 << 8);
                        break;
                    }
                    case 240: {
                        switch (i) {
                            case 240:
                            case 247: {
                                int length = (int)this.readVarInt();
                                byte[] data = new byte[length];
                                this.read(data);
                                SysexMessage o = new SysexMessage();
                                o.setMessage(i, data, length);
                                message = o;
                                break;
                            }
                            case 255: {
                                int unsigned2 = this.readUnsigned();
                                int length2 = (int)this.readVarInt();
                                byte[] data2;
                                try {
                                    data2 = new byte[length2];
                                }
                                catch (OutOfMemoryError cause) {
                                    throw new IOException("Meta length too big", cause);
                                }
                                this.read(data2);
                                MetaMessage o2 = new MetaMessage();
                                o2.setMessage(unsigned2, data2, length2);
                                message = o2;
                                if (unsigned2 == 47) {
                                    n = 1;
                                    break;
                                }
                                break;
                            }
                            default: {
                                throw new InvalidMidiDataException("Invalid status byte: " + i);
                            }
                        }
                        break;
                    }
                    default: {
                        throw new InvalidMidiDataException("Invalid status byte: " + i);
                    }
                }
                track.add(new MidiEvent(message, tick));
            }
        }
        catch (ArrayIndexOutOfBoundsException ex) {
            throw new EOFException("invalid MIDI file");
        }
    }
}
